% This code generates the reduced order controllers, runs a simulation on
% all of them using different solver types and step-sizes, records the time
% spent on each simulation, then saves and plots the results.

new_order_it = 3:18;
solver_type_it = {'ode1','ode2','ode3','ode5','ode8','ode14x'};
step_time_it = [1e-4, 1e-5, 4e-5, 1e-6];

resSimTime = zeros(size(new_order_it,2), size(solver_type_it,2), size(step_time_it,2));
resRMSx = Inf*ones(size(new_order_it,2), size(solver_type_it,2), size(step_time_it,2));
resRMSy = Inf*ones(size(new_order_it,2), size(solver_type_it,2), size(step_time_it,2));
resDT = zeros(size(new_order_it,2),1);
resWT = zeros(size(new_order_it,2),1);
resEig = zeros(size(new_order_it,2),18);

for new_order_index = 1:size(new_order_it,2);%[7,10,13,17,18];%[4,7,9,10]%2:17
    % Step response full order controller
    K_r = K;
    simout = sim('controller_xpc','StopTime', '.02');
    y = simout.get('yout');
    t = simout.get('tout');
    
    % Generate reduced order controller
    new_order = new_order_it(new_order_index);
    if new_order == 18
        K_r = K;
    else
        [K_bal,g] = balreal(K);
        elim = [ zeros(new_order,1); ones(size(g,1)-new_order,1) ];
        K_r = modred(K_bal,logical(elim));

        L_r = G*K_r;
        T_r = (eye(2) + L_r)\L_r;
        S_r = eye(2) - T_r;

        dT = T-T_r;
        resDT(new_order_index,1) = norm(pade(dT,5),inf);
        resWT(new_order_index,1) = norm(pade(W_O*T_r,5),inf);
        eigVals = eig(K_r);
        resEig(new_order_index,1) = eigVals(1);
        disp(['dT: ',num2str( norm(dT,inf), '%3.2e' )]);
    end
    
    eigVals = sort(eig(K_r))
    resEig(new_order_index,1:size(eigVals,1)) = eigVals;
        
    solver_type = char(solver_type_it(1));
    set_param('hinf_ms_controller_stripped','Solver',solver_type);
    
    h=figure(9);
    set(h,'DefaultAxesColorOrder',[0 0 1;0 .5 0;0 0 1;0 .5 0])
    hb=sigmaplot(S,'b',S_r,'g',T,'b',T_r,'g');
    setoptions(hb,'FreqUnits','Hz'); legend('S, T','S_r, T_r','Location','SouthWest');
    title([num2str(new_order),'th order']); xlim([1,3e4]); ylim([-35,10]); grid on;
    export_plot(['experiment_04_S_T_order_', num2str(new_order),'.pdf']);

    Step response reduced order controller
    simout = sim('sim_closed_loop','StopTime', '.02');
    y_r = simout.get('yout');
    t_r = simout.get('tout');
    
    plot(t,y(:,1),t_r,y_r(:,1)); % step in r1
    hold on;
    plot( t,y(:,2),t_r,y_r(:,2));
    hold off;
    legend('18th order (full)',[num2str(new_order),'th order'], 'Location','Best'); xlabel('time (s)'); ylabel('output (V)'); grid on;
    title(['Step response']);
    export_plot(['experiment_04_step_sim_order_', num2str(new_order),'.pdf']);
         
    % Run simulation for various solvers and step time
    for solver_type_index = 1:size(solver_type_it,2);
    solver_type = solver_type_it(solver_type_index);
    for step_time_index = 1:size(step_time_it,2);
        step_time = step_time_it(step_time_index);

        % Simulation
        Dt_step = step_time;
        disp(['Order: ',num2str(new_order),'  Solver: ',char(solver_type),'  StepTime: ',num2str(Dt_step)]);
        %set_param('sim_closed_loop','Solver','ode14x');
        try
            t0 = tic;
            simout = sim('sim_closed_loop','StopTime', '.2','Solver',char(solver_type));
            elapsedTime = toc(t0);
            y = simout.get('yout');
            
            resSimTime(new_order_index, solver_type_index, step_time_index) = elapsedTime;
            resRMSx(new_order_index, solver_type_index, step_time_index) = rms(y(:,1));
            resRMSy(new_order_index, solver_type_index, step_time_index) = rms(y(:,2));
            
            t = simout.get('tout');
            plot(t,y);
            hold on

            disp(['RMS x: ',num2str( rms( y(:,1) )),'  y: ',num2str( rms( y(:,2) ))]);
        catch err
            disp(['Unstable Error:',err.identifier]);
        end
        disp(resSimTime(new_order_index, solver_type_index, step_time_index));
    end
    end
end

hold off;
set_param('hinf_ms_controller_stripped','Solver',char(solver_type));

%%
GAM
norm(pade(W_O*T,5),Inf)

%%
figure1 = figure(6);
axes1 = axes('Parent',figure1,'YScale','log','YMinorTick','on',...
    'YMinorGrid','on',...
    'XTick',2:17);
xlim(axes1,[1.5 17.5]); box(axes1,'on');
grid(axes1,'on');
hold(axes1,'all');
xlabel('Model Order'); ylabel('||T-T_r||_\infty');
set(gcf,'Position',[680   558   460   420]);

bar(new_order_it,resDT,'BaseValue',0.001);
export_plot('experiment_04_hinf_error_norm_bar.pdf',figure1);

%%
figure1 = figure(6);
axes1 = axes('Parent',figure1,'YScale','log','YMinorTick','on',...
    'YMinorGrid','on',...
    'XTick',2:17);
xlim(axes1,[1.5 17.5]); box(axes1,'on');
grid(axes1,'on');
hold(axes1,'all');
xlabel('Model Order'); ylabel('||W T_r||_\infty');
set(gcf,'Position',[680   558   460   420]);

bar(new_order_it,resWT,'BaseValue',.1);
export_plot('experiment_04_W_x_T_robustness.pdf',figure1);
